@@ -342,16 +342,9 @@ get_mod(<<"failure">>,
         <<"urn:ietf:params:xml:ns:xmpp-sasl">>) ->
     rfc6120;
 get_mod(<<"PRIVATE">>, <<"vcard-temp">>) -> xep0054;
-get_mod(<<"options">>,
-        <<"http://jabber.org/protocol/pubsub">>) ->
-    xep0060;
 get_mod(<<"pubsub">>,
         <<"http://jabber.org/protocol/pubsub">>) ->
     xep0060;
-get_mod(<<"disable">>, <<"urn:xmpp:carbons:2">>) ->
-    xep0280;
-get_mod(<<"mix">>, <<"urn:xmpp:mix:presence:0">>) ->
-    xep0403;
 get_mod(<<"unsupported-method">>,
         <<"http://jabber.org/protocol/compress">>) ->
     xep0138;
@@ -614,6 +607,7 @@ get_mod(<<"size">>,
     xep0234;
 get_mod(<<"hash">>, <<"urn:xmpp:scram-upgrade:0">>) ->
     xep0480;
+get_mod(<<"fast">>, <<"urn:xmpp:fast:0">>) -> xep0484;
 get_mod(<<"query">>,
         <<"http://jabber.org/protocol/stats">>) ->
     xep0039;
@@ -650,9 +644,6 @@ get_mod(<<"no-store">>, <<"urn:xmpp:hints">>) ->
 get_mod(<<"nick">>,
         <<"http://jabber.org/protocol/nick">>) ->
     xep0172;
-get_mod(<<"hats">>,
-        <<"xmpp:prosody.im/protocol/hats:1">>) ->
-    xep0317;
 get_mod(<<"moderate">>,
         <<"urn:xmpp:message-moderate:0">>) ->
     xep0425;
@@ -757,6 +748,7 @@ get_mod(<<"metadata">>,
     xep0084;
 get_mod(<<"x509-cert">>, <<"urn:xmpp:x509:0">>) ->
     xep0417;
+get_mod(<<"hat">>, <<"urn:xmpp:hats:0">>) -> xep0317;
 get_mod(<<"jid">>, <<"urn:xmpp:jidprep:0">>) -> xep0328;
 get_mod(<<"password">>,
         <<"http://jabber.org/protocol/muc#owner">>) ->
@@ -878,6 +870,7 @@ get_mod(<<"CELL">>, <<"vcard-temp">>) -> xep0054;
 get_mod(<<"stream:stream">>,
         <<"jabber:component:accept">>) ->
     rfc6120;
+get_mod(<<"token">>, <<"urn:xmpp:fast:0">>) -> xep0484;
 get_mod(<<"email">>, <<"jabber:iq:register">>) ->
     xep0077;
 get_mod(<<"event">>,
@@ -945,8 +938,6 @@ get_mod(<<"sm">>, <<"urn:xmpp:sm:3">>) -> xep0198;
 get_mod(<<"fetch">>,
         <<"http://jabber.org/protocol/offline">>) ->
     xep0013;
-get_mod(<<"participant">>, <<"urn:xmpp:mix:core:0">>) ->
-    xep0369;
 get_mod(<<"failed-transport">>,
         <<"urn:xmpp:jingle:1">>) ->
     xep0166;
@@ -1067,6 +1058,7 @@ get_mod(<<"authorization-identifier">>,
         <<"urn:xmpp:sasl:2">>) ->
     xep0388;
 get_mod(<<"bind">>, <<"urn:xmpp:bind:0">>) -> xep0386;
+get_mod(<<"bidi">>, <<"urn:xmpp:bidi">>) -> xep0288;
 get_mod(<<"MODEM">>, <<"vcard-temp">>) -> xep0054;
 get_mod(<<"ORGUNIT">>, <<"vcard-temp">>) -> xep0054;
 get_mod(<<"UID">>, <<"vcard-temp">>) -> xep0054;
@@ -1079,6 +1071,8 @@ get_mod(<<"TEL">>, <<"vcard-temp">>) -> xep0054;
 get_mod(<<"prefs">>, <<"urn:xmpp:mam:tmp">>) -> xep0313;
 get_mod(<<"captcha">>, <<"urn:xmpp:captcha">>) ->
     xep0158;
+get_mod(<<"mechanism">>, <<"urn:xmpp:fast:0">>) ->
+    xep0484;
 get_mod(<<"identity">>,
         <<"http://jabber.org/protocol/disco#info">>) ->
     xep0030;
@@ -1139,6 +1133,8 @@ get_mod(<<"invalid-subid">>,
 get_mod(<<"not-in-roster-group">>,
         <<"http://jabber.org/protocol/pubsub#errors">>) ->
     xep0060;
+get_mod(<<"request-token">>, <<"urn:xmpp:fast:0">>) ->
+    xep0484;
 get_mod(<<"unsupported-encoding">>,
         <<"urn:ietf:params:xml:ns:xmpp-streams">>) ->
     rfc6120;
@@ -1315,6 +1311,7 @@ get_mod(<<"delegation">>,
     xep0355;
 get_mod(<<"expired">>, <<"urn:xmpp:jingle:1">>) ->
     xep0166;
+get_mod(<<"hats">>, <<"urn:xmpp:hats:0">>) -> xep0317;
 get_mod(<<"user-agent">>, <<"urn:xmpp:sasl:2">>) ->
     xep0388;
 get_mod(<<"bad-request">>,
@@ -1452,9 +1449,6 @@ get_mod(<<"destroy">>,
 get_mod(<<"participant">>, <<"urn:xmpp:mix:core:1">>) ->
     xep0369;
 get_mod(<<"id">>, <<"jabber:x:event">>) -> xep0022;
-get_mod(<<"hat">>,
-        <<"xmpp:prosody.im/protocol/hats:1">>) ->
-    xep0317;
 get_mod(<<"occupant-id">>,
         <<"urn:xmpp:occupant-id:0">>) ->
     xep0421;
@@ -1740,6 +1734,13 @@ get_mod(<<"internal-server-error">>,
 get_mod(<<"put">>,
         <<"eu:siacs:conversations:http:upload">>) ->
     xep0363;
+get_mod(<<"options">>,
+        <<"http://jabber.org/protocol/pubsub">>) ->
+    xep0060;
+get_mod(<<"disable">>, <<"urn:xmpp:carbons:2">>) ->
+    xep0280;
+get_mod(<<"mix">>, <<"urn:xmpp:mix:presence:0">>) ->
+    xep0403;
 get_mod(Name, XMLNS) ->
     xmpp_codec_external:lookup(Name, XMLNS).
 
@@ -1826,8 +1827,10 @@ get_mod({carbons_enable}) -> xep0280;
 get_mod({carbons_private}) -> xep0280;
 get_mod({mix_roster_channel, _}) -> xep0405;
 get_mod({fasten_external, _}) -> xep0422;
+get_mod({s2s_bidi_feature}) -> xep0288;
 get_mod({shim, _}) -> xep0131;
 get_mod({mam_archived, _, _}) -> xep0313;
+get_mod({mix_participant, _, _}) -> xep0369;
 get_mod({delegated, _, _}) -> xep0355;
 get_mod({text, _, _}) -> xep0234;
 get_mod({muc_hat, _, _}) -> xep0317;
@@ -1839,6 +1842,7 @@ get_mod({ps_subscribe, _, _}) -> xep0060;
 get_mod({adhoc_actions, _, _, _, _}) -> xep0050;
 get_mod({push_disable, _, _}) -> xep0357;
 get_mod({push_notification, _}) -> xep0357;
+get_mod({fast_request_token, _}) -> xep0484;
 get_mod({disco_item, _, _, _}) -> xep0030;
 get_mod({register,
          _,
@@ -1866,38 +1870,18 @@ get_mod({register,
     xep0077;
 get_mod({service, _, _, _, _, _, _, _, _, _, _, _}) ->
     xep0215;
+get_mod({fast, _, _, _, _}) -> xep0484;
 get_mod({version, _, _, _}) -> xep0092;
 get_mod({muc_subscription, _, _, _}) -> p1_mucsub;
 get_mod({sasl2_authenticaton, _, _, _}) -> xep0388;
 get_mod({sasl2_authenticate, _, _, _, _}) -> xep0388;
 get_mod({sasl2_continue, _, _, _, _}) -> xep0388;
 get_mod({s2s_bidi}) -> xep0288;
-get_mod({vcard_tel,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _,
-         _}) ->
-    xep0054;
 get_mod({vcard_photo, _, _, _}) -> xep0054;
 get_mod({ps_publish, _, _}) -> xep0060;
-get_mod({mam_query, _, _, _, _, _, _, _, _, _}) ->
-    xep0313;
-get_mod({search_item, _, _, _, _, _}) -> xep0055;
 get_mod({xcaptcha, _}) -> xep0158;
 get_mod({avatar_data, _}) -> xep0084;
-get_mod({hash, _, _}) -> xep0300;
 get_mod({fasten_apply_to, _, _, _}) -> xep0422;
-get_mod({sasl2_task_data, _}) -> xep0388;
 get_mod({vcard_key, _, _}) -> xep0054;
 get_mod({sm_r, _}) -> xep0198;
 get_mod({mix_join, _, _, _, _, _}) -> xep0369;
@@ -1977,7 +1961,6 @@ get_mod({sasl_upgrade, _}) -> xep0480;
 get_mod({stanza_error, _, _, _, _, _}) -> rfc6120;
 get_mod({sm_resumed, _, _, _}) -> xep0198;
 get_mod({offline_item, _, _}) -> xep0013;
-get_mod({mix_participant, _, _, _}) -> xep0369;
 get_mod({db_verify, _, _, _, _, _, _}) -> xep0220;
 get_mod({upload_slot, _, _, _}) -> xep0363;
 get_mod({jingle_ibb_transport, _, _, _}) -> xep0261;
@@ -1989,6 +1972,7 @@ get_mod({mix_client_leave, _, _, _}) -> xep0405;
 get_mod({thumbnail, _, _, _, _}) -> xep0264;
 get_mod({delegation, _, _}) -> xep0355;
 get_mod({x509_challenge, _, _, _}) -> xep0417;
+get_mod({fast_token, _, _}) -> xep0484;
 get_mod({sm_enabled, _, _, _, _, _}) -> xep0198;
 get_mod({privilege_perm, _, _, _, _}) -> xep0356;
 get_mod({message_retracted, _, _, _, _, _}) -> xep0424;
@@ -2122,4 +2106,25 @@ get_mod({ps_items, _, _, _, _, _, _}) -> xep0060;
 get_mod({muc_subscribe, _, _, _, _}) -> p1_mucsub;
 get_mod({feature_csi}) -> xep0352;
 get_mod({ibb_open, _, _, _}) -> xep0047;
+get_mod({vcard_tel,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _,
+         _}) ->
+    xep0054;
+get_mod({mam_query, _, _, _, _, _, _, _, _, _}) ->
+    xep0313;
+get_mod({search_item, _, _, _, _, _}) -> xep0055;
+get_mod({hash, _, _}) -> xep0300;
+get_mod({sasl2_task_data, _}) -> xep0388;
 get_mod(Record) -> xmpp_codec_external:lookup(Record).
