@@ -75,7 +75,7 @@
 			wait_for_starttls | wait_for_sasl_request |
 			wait_for_sasl_response | wait_for_bind |
 			established | disconnected | wait_for_sasl2_response |
-			wait_for_sasl2_task_data | wait_for_sasl2_task_data.
+			wait_for_sasl2_next | wait_for_sasl2_task_data.
 -type stop_reason() :: {stream, reset | {in | out, stream_error()}} |
 		       {tls, inet:posix() | atom() | binary()} |
 		       {socket, inet:posix() | atom()} |
@@ -105,6 +105,8 @@
 -callback get_password_fun(xmpp_sasl:mechanism(), state()) -> fun().
 -callback check_password_fun(xmpp_sasl:mechanism(), state()) -> fun().
 -callback check_password_digest_fun(xmpp_sasl:mechanism(), state()) -> fun().
+-callback get_fast_tokens_fun(xmpp_sasl:mechanism(), state()) -> fun().
+-callback fast_mechanisms(state()) -> [xmpp_sasl:mechanism()].
 -callback bind(binary(), state()) -> {ok, state()} | {error, stanza_error(), state()}.
 -callback compress_methods(state()) -> [binary()].
 -callback tls_options(state()) -> [proplists:property()].
@@ -139,6 +141,8 @@
 		     get_password_fun/2,
 		     check_password_fun/2,
 		     check_password_digest_fun/2,
+		     get_fast_tokens_fun/2,
+		     fast_mechanisms/1,
 		     bind/2,
 		     compress_methods/1,
 		     tls_options/1,
@@ -209,8 +213,8 @@ send_ws_ping(#{owner := Owner, socket := Sock,
     case xmpp_socket:send_ws_ping(Sock) of
 	ok ->
 	    State;
-	{error, Why} ->
-	    process_stream_end({socket, Why}, State)
+	{error, _} = Err ->
+	    process_stream_end(convert_socket_error(Err), State)
     end;
 send_ws_ping(State) ->
     State.
@@ -445,11 +449,8 @@ handle_info({tcp, _, Data}, #{socket := Socket} = State) ->
       case xmpp_socket:recv(Socket, Data) of
 	  {ok, NewSocket} ->
 	      State#{socket => NewSocket};
-	  {error, Reason} when is_atom(Reason) ->
-	      process_stream_end({socket, Reason}, State);
-	  {error, Reason} ->
-	      %% TODO: make fast_tls return atoms
-	      process_stream_end({tls, Reason}, State)
+	  {error, _} = Err ->
+	      process_stream_end(convert_socket_error(Err), State)
       end);
 % Skip new tcp messages after socket get removed from state
 handle_info({tcp, _, _}, State) ->
@@ -540,6 +541,12 @@ noreply(#{stream_timeout := {MSecs, StartTime}} = State) ->
 is_disconnected(#{stream_state := StreamState}) ->
     StreamState == disconnected.
 
+convert_socket_error({error, Reason}) when is_atom(Reason) ->
+	{socket, Reason};
+convert_socket_error({error, Reason}) ->
+	%% TODO: make fast_tls return atoms
+	{tls, Reason}.
+
 -spec process_invalid_xml(state(), fxml:xmlel(), term()) -> state().
 process_invalid_xml(#{lang := MyLang} = State, El, Reason) ->
     case xmpp:is_stanza(El) of
@@ -927,8 +934,10 @@ process_starttls_failure(Why, State) ->
     end.
 
 -spec init_channel_bindings(state()) -> state().
-init_channel_bindings(#{sasl_channel_bindings := _} = State) ->
+init_channel_bindings(#{sasl_channel_bindings := #{}} = State) ->
     State;
+init_channel_bindings(#{stream_encrypted := false} = State) ->
+    State#{sasl_channel_bindings => not_available};
 init_channel_bindings(#{socket := Socket} = State) ->
 	R1 = case xmpp_socket:get_tls_last_message(Socket, peer) of
 		 {ok, Data} ->
@@ -976,9 +985,9 @@ process_sasl_request(#sasl_auth{mechanism = Mech, text = ClientIn},
 			     end
 		     catch _:{?MODULE, undef} -> Mechs
 		     end,
-	    SASLState = xmpp_sasl:server_new(LServer, GetPW, CheckPW, CheckPWDigest),
+	    SASLState = xmpp_sasl:server_new(LServer, GetPW, CheckPW, CheckPWDigest, undefined),
 	    CB = maps:get(sasl_channel_bindings, State1, none),
-	    Res = xmpp_sasl:server_start(SASLState, Mech, ClientIn, CB, Mechs2),
+	    Res = xmpp_sasl:server_start(SASLState, Mech, ClientIn, CB, Mechs2, undefined),
 	    process_sasl_result(Res, disable_sasl2(State1#{sasl_state => SASLState}));
 	false ->
 	    process_sasl_result({error, unsupported_mechanism, <<"">>}, disable_sasl2(State1))
@@ -1064,7 +1073,11 @@ process_sasl2_request(#sasl2_authenticate{mechanism = Mech, initial_response = C
 					  user_agent = UA} = Pkt,
 		     #{lserver := LServer} = State) ->
     State1 = State#{sasl_mech => Mech},
+    FastMechs = try callback(fast_mechanisms, State)
+		catch _:{?MODULE, undef} -> []
+		end,
     Mechs = get_sasl_mechanisms(State1),
+    MechsAll = Mechs ++ FastMechs,
     UAId = case UA of
 	       #sasl2_user_agent{id = ID} when ID /= <<>> ->
 		   ID;
@@ -1075,7 +1088,7 @@ process_sasl2_request(#sasl2_authenticate{mechanism = Mech, initial_response = C
 		     #sasl2_authenticate{sub_els = SubEls} -> SubEls
 		 catch _:{xmpp_codec, _} -> []
 		 end,
-    case lists:member(Mech, Mechs) of
+    case lists:member(Mech, MechsAll) of
 	true when Mech == <<"EXTERNAL">> ->
 	    Res = case xmpp_stream_pkix:authenticate(State1, ClientIn) of
 		      {ok, Peer} ->
@@ -1089,6 +1102,7 @@ process_sasl2_request(#sasl2_authenticate{mechanism = Mech, initial_response = C
 	    GetPW = get_password_fun(Mech, State1),
 	    CheckPW = check_password_fun(Mech, State1),
 	    CheckPWDigest = check_password_digest_fun(Mech, State1),
+	    GetFastTokens = get_fast_tokens_fun(Mech, State1),
 	    Mechs2 = try callback(sasl_options, State) of
 			 Opts ->
 			     case lists:keyfind(scram_downgrade_protection, 1, Opts) of
@@ -1097,9 +1111,10 @@ process_sasl2_request(#sasl2_authenticate{mechanism = Mech, initial_response = C
 			     end
 		     catch _:{?MODULE, undef} -> Mechs
 		     end,
-	    SASLState = xmpp_sasl:server_new(LServer, GetPW, CheckPW, CheckPWDigest),
+	    SASLState = xmpp_sasl:server_new(LServer, GetPW, CheckPW,
+					     CheckPWDigest, GetFastTokens),
 	    CB = maps:get(sasl_channel_bindings, State1, none),
-	    Res = xmpp_sasl:server_start(SASLState, Mech, ClientIn, CB, Mechs2),
+	    Res = xmpp_sasl:server_start(SASLState, Mech, ClientIn, CB, Mechs2, UAId),
 	    process_sasl2_result(Res, State1#{sasl_state => SASLState,
 					      sasl2_inline_els => SaslInline,
 					      sasl2_ua_id => UAId});
@@ -1133,8 +1148,10 @@ process_sasl2_success(Props, ServerOut,
 	process_sasl2_failure(not_authorized, User, State);
 	true ->
 	    AuthModule = proplists:get_value(auth_module, Props),
-	    State1 = try callback(handle_auth_success, User, Mech, AuthModule, State)
-		     catch _:{?MODULE, undef} -> State
+	    ExtraAuthInfo = proplists:get_value(extra_info, Props),
+	    State0 = State#{sasl2_axtra_auth_info => ExtraAuthInfo},
+	    State1 = try callback(handle_auth_success, User, Mech, AuthModule, State0)
+		     catch _:{?MODULE, undef} -> State0
 		     end,
 	    case is_disconnected(State1) of
 		true -> State1;
@@ -1209,7 +1226,7 @@ process_sasl2_post_success(NewEls, Results, User, ServerOut,
 		false ->
 		    map_remove_keys(State5, [sasl2_stream_from, sasl2_inline_els,
 					     sasl2_ua_id, sasl_state, sasl_mech,
-					     sasl_channel_bindings])
+					     sasl_channel_bindings, sasl2_extra_auth_info])
 	    end
     end.
 
@@ -1318,30 +1335,31 @@ send_features(#{stream_version := {1,0},
 	(not Encrypted) andalso TLSRequired ->
 	    send_pkt(State, #stream_features{sub_els = get_tls_feature(State)});
 	true ->
-	    {Features, State2} =
+	    State2 = case AllowUnencryptedSasl2 orelse Encrypted of
+			 true -> init_channel_bindings(State);
+			 _ -> State
+		     end,
+	    {Features, State3} =
 	    case {Encrypted, Sasl2, AllowUnencryptedSasl2, TLSAvailable} of
 		{false, true, true, true} ->
-		    {get_tls_feature(State) ++ get_sasl2_feature(State),
-		     init_channel_bindings(State)};
+		    {get_tls_feature(State2) ++ get_sasl2_feature(State2), State2};
 		{_, true, _, _} when Encrypted; AllowUnencryptedSasl2 ->
-		    {get_sasl2_feature(State), init_channel_bindings(State)};
+		    {get_sasl2_feature(State2), State2};
 		{false, true, false, false} ->
-		    {[], disable_sasl2(State)};
+		    {[], disable_sasl2(State2)};
 		{false, _, _, true} ->
-		    {get_tls_feature(State), State};
-		{true, _, _, _} ->
-		    {[], init_channel_bindings(State)};
+		    {get_tls_feature(State2), State2};
 		_ ->
-		    {[], State}
+		    {[], State2}
 	    end,
 	    Features2 =
-		get_sasl_feature(State2) ++
+		get_sasl_feature(State3) ++
 		Features ++
-		get_compress_feature(State2) ++
-		get_bind_feature(State2) ++
-		get_session_feature(State2) ++
-		get_other_features(State2),
-	    send_pkt(State2, #stream_features{sub_els = Features2})
+		get_compress_feature(State3) ++
+		get_bind_feature(State3) ++
+		get_session_feature(State3) ++
+		get_other_features(State3),
+	    send_pkt(State3, #stream_features{sub_els = Features2})
     end;
 send_features(State) ->
     %% clients and servers from stone age
@@ -1371,6 +1389,12 @@ check_password_digest_fun(Mech, State) ->
     catch _:{?MODULE, undef} -> fun(_, _, _, _, _) -> {false, undefined} end
     end.
 
+-spec get_fast_tokens_fun(xmpp_sasl:mechanism(), state()) -> fun().
+get_fast_tokens_fun(Mech, State) ->
+    try callback(get_fast_tokens_fun, Mech, State)
+    catch _:{?MODULE, undef} -> fun(_, _) -> [] end
+    end.
+
 -spec get_sasl_mechanisms(state()) -> [xmpp_sasl:mechanism()].
 get_sasl_mechanisms(#{stream_encrypted := Encrypted,
 		      xmlns := NS} = State) ->
@@ -1394,6 +1418,7 @@ get_sasl_feature(#{stream_authenticated := false,
 	    [#sasl_mechanisms{list = Mechs}] ++
 	    case maps:get(sasl_channel_bindings, State, none) of
 		none -> [];
+		not_available -> [];
 		Bindings -> [#sasl_channel_binding{bindings = maps:keys(Bindings)}]
 	    end;
 	true ->
@@ -1557,7 +1582,7 @@ send_header(#{stream_id := StreamID,
 		    stream_header_sent => true},
     case socket_send(State1, StreamStart) of
 	ok -> State1;
-	{error, Why} -> process_stream_end({socket, Why}, State1)
+	{error, _} = Err -> process_stream_end(convert_socket_error(Err), State1)
     end;
 send_header(State, _) ->
     State.
